@@ -28,6 +28,7 @@ from glue.plugins.wcs_autolinking.wcs_autolinking import WCSLink, IncompatibleWC
 from glue.core.message import (DataCollectionAddMessage,
                                DataCollectionDeleteMessage,
                                SubsetCreateMessage,
+                               SubsetUpdateMessage,
                                SubsetDeleteMessage)
 from glue.core.state_objects import State
 from glue.core.subset import Subset, RangeSubsetState, RoiSubsetState
@@ -258,6 +259,12 @@ class Application(VuetifyTemplate, HubListener):
         # Add a fitted_models dictionary that the helpers (or user) can access
         self.fitted_models = {}
 
+        # Internal cache so we don't have to keep calling get_object for the same Data.
+        # Key should be (data_label, statistic) and value the translated object.
+        self._get_object_cache = {}
+        self.hub.subscribe(self, SubsetUpdateMessage,
+                           handler=lambda msg: self._clear_object_cache(msg.subset.label))
+
         # Add new and inverse colormaps to Glue global state. Also see ColormapRegistry in
         # https://github.com/glue-viz/glue/blob/main/glue/config.py
         new_cms = (['Rainbow', cm.rainbow],
@@ -405,7 +412,11 @@ class Application(VuetifyTemplate, HubListener):
 
         dc = self.data_collection
         # This will need to be changed for cubeviz to support multiple cubes
-        ref_data = dc[reference_data] if reference_data else dc[0]
+        default_refdata_index = 0
+        if self.config == 'mosviz':
+            # In Mosviz, first data is always MOS Table. Use the next data
+            default_refdata_index = 1
+        ref_data = dc[reference_data] if reference_data else dc[default_refdata_index]
         linked_data = dc[data_to_be_linked] if data_to_be_linked else dc[-1]
 
         if 'Trace' in linked_data.meta:
@@ -1159,16 +1170,17 @@ class Application(VuetifyTemplate, HubListener):
         viewer has not yet loaded data, or e.g. ``require_spectrum_viewer``
         to require that the viewer supports spectrum visualization.
         """
+        from jdaviz.configs.imviz.plugins.viewers import ImvizImageView
         from jdaviz.configs.specviz.plugins.viewers import SpecvizProfileView
-        from jdaviz.configs.specviz2d.plugins import SpectralExtraction
         from jdaviz.configs.cubeviz.plugins.viewers import CubevizProfileView, CubevizImageView
         from jdaviz.configs.mosviz.plugins.viewers import (
             MosvizTableViewer, MosvizProfile2DView
         )
 
         spectral_viewers = (SpecvizProfileView, CubevizProfileView)
+        spectral_2d_viewers = (MosvizProfile2DView, )
         table_viewers = (MosvizTableViewer, )
-        image_viewers = (MosvizProfile2DView, CubevizImageView, SpectralExtraction)
+        image_viewers = (ImvizImageView, CubevizImageView)
         flux_viewers = (CubevizImageView, )
 
         for vid in self._viewer_store:
@@ -1180,6 +1192,9 @@ class Application(VuetifyTemplate, HubListener):
             if require_spectrum_viewer:
                 if isinstance(self._viewer_store[vid], spectral_viewers) and is_returnable:
                     return viewer_item['reference']
+            elif require_spectrum_2d_viewer:
+                if isinstance(self._viewer_store[vid], spectral_2d_viewers) and is_returnable:
+                    return viewer_item['reference']
             elif require_table_viewer:
                 if isinstance(self._viewer_store[vid], table_viewers) and is_returnable:
                     return viewer_item['reference']
@@ -1478,6 +1493,14 @@ class Application(VuetifyTemplate, HubListener):
         data_item = self._create_data_item(msg.data)
         self.state.data_items.append(data_item)
 
+    def _clear_object_cache(self, data_label=None):
+        if data_label is None:
+            self._get_object_cache.clear()
+        else:
+            # keys are (data_label, statistic) tuples
+            self._get_object_cache = {k: v for k, v in self._get_object_cache.items()
+                                      if k[0] != data_label}
+
     def _on_data_deleted(self, msg):
         """
         Callback for when data is removed from the internal ``DataCollection``.
@@ -1493,6 +1516,8 @@ class Application(VuetifyTemplate, HubListener):
             if data_item['name'] == msg.data.label:
                 self.state.data_items.remove(data_item)
 
+        self._clear_object_cache(msg.data.label)
+
     @staticmethod
     def _create_data_item(data):
         ndims = len(data.shape)
