@@ -3,7 +3,7 @@ import pytest
 from astropy import units as u
 from astropy.coordinates import SkyCoord, Angle
 from astropy.utils.data import get_pkg_data_filename
-from photutils import CircularAperture, SkyCircularAperture
+from photutils.aperture import CircularAperture, SkyCircularAperture
 from regions import (PixCoord, CircleSkyRegion, RectanglePixelRegion, CirclePixelRegion,
                      EllipsePixelRegion, PointPixelRegion, PointSkyRegion, PolygonPixelRegion,
                      CircleAnnulusSkyRegion, Regions)
