@@ -1,69 +1,22 @@
 import numpy as np
-from astropy import units as u
 from glue.core import BaseData
 from glue_jupyter.bqplot.image import BqplotImageView
-from glue_jupyter.bqplot.profile import BqplotProfileView
 from glue_jupyter.table import TableViewer
 from specutils import Spectrum1D
 
 from jdaviz.core.events import (AddDataToViewerMessage,
                                 RemoveDataFromViewerMessage,
                                 TableClickMessage)
+from jdaviz.core.helpers import data_has_valid_wcs
 from jdaviz.core.registries import viewer_registry
 from jdaviz.core.freezable_state import FreezableBqplotImageViewerState
 from jdaviz.configs.default.plugins.viewers import JdavizViewerMixin
 from jdaviz.configs.imviz.helper import layer_is_image_data
 
-__all__ = ['MosvizProfileView', 'MosvizImageView', 'MosvizProfile2DView',
+__all__ = ['MosvizImageView', 'MosvizProfile2DView',
            'MosvizTableViewer']
 
 
-@viewer_registry("mosviz-profile-viewer", label="Profile 1D (Mosviz)")
-class MosvizProfileView(JdavizViewerMixin, BqplotProfileView):
-    default_class = Spectrum1D
-
-    # categories: zoom resets, zoom, pan, subset, select tools, shortcuts
-    tools_nested = [
-                    ['jdaviz:homezoom', 'jdaviz:prevzoom'],
-                    ['jdaviz:boxzoom', 'jdaviz:xrangezoom'],
-                    ['jdaviz:panzoom', 'jdaviz:panzoom_x', 'jdaviz:panzoom_y'],
-                    ['bqplot:xrange'],
-                    ['jdaviz:sidebar_plot', 'jdaviz:sidebar_export']
-                ]
-
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._subscribe_to_layers_update()
-
-    def data(self, cls=None):
-        return [layer_state.layer.get_object(cls=cls or self.default_class)
-                for layer_state in self.state.layers
-                if hasattr(layer_state, 'layer') and
-                isinstance(layer_state.layer, BaseData)]
-
-    def set_plot_axes(self):
-        data = self.data()[0]
-        # Set axes labels for the spectrum viewer
-        spectral_axis_unit_type = str(data.spectral_axis.unit.physical_type).title()
-        # flux_unit_type = data.flux.unit.physical_type.title()
-        flux_unit_type = "Flux density"
-
-        if data.spectral_axis.unit.is_equivalent(u.m):
-            spectral_axis_unit_type = "Wavelength"
-        elif data.spectral_axis.unit.is_equivalent(u.pixel):
-            spectral_axis_unit_type = "pixel"
-
-        label_0 = f"{spectral_axis_unit_type} [{data.spectral_axis.unit.to_string()}]"
-        self.figure.axes[0].label = label_0
-        self.figure.axes[1].label = f"{flux_unit_type} [{data.flux.unit.to_string()}]"
-
-        # Make it so y axis label is not covering tick numbers.
-        self.figure.axes[1].label_offset = "-50"
-
-        # Set Y-axis to scientific notation
-        self.figure.axes[1].tick_format = '0.1e'
-
-
 @viewer_registry("mosviz-image-viewer", label="Image 2D (Mosviz)")
 class MosvizImageView(JdavizViewerMixin, BqplotImageView):
     # categories: zoom resets, zoom, pan, subset, select tools, shortcuts
@@ -81,6 +34,81 @@ class MosvizImageView(JdavizViewerMixin, BqplotImageView):
         super().__init__(*args, **kwargs)
         self._subscribe_to_layers_update()
 
+        self.label_mouseover = None
+        self.add_event_callback(self.on_mouse_or_key_event, events=['mousemove', 'mouseenter',
+                                                                    'mouseleave'])
+
+    def on_mouse_or_key_event(self, data):
+
+        # Find visible layers
+        visible_layers = [layer for layer in self.state.layers
+                          if (layer.visible and layer_is_image_data(layer.layer))]
+
+        if len(visible_layers) == 0:
+            return
+
+        # TODO: mosviz/helper.py just calls imview.layers[0] so will this work?
+        active_layer = visible_layers[-1]
+
+        if self.label_mouseover is None:
+            if 'g-coords-info' in self.session.application._tools:
+                self.label_mouseover = self.session.application._tools['g-coords-info']
+            else:  # pragma: no cover
+                return
+
+        if data['event'] == 'mousemove':
+            # Display the current cursor coordinates (both pixel and world) as
+            # well as data values. For now we use the first dataset in the
+            # viewer for the data values.
+
+            # Extract first dataset from visible layers and use this for coordinates - the choice
+            # of dataset shouldn't matter if the datasets are linked correctly
+            image = active_layer.layer
+            self.label_mouseover.icon = self.jdaviz_app.state.layer_icons.get(active_layer.layer.label)  # noqa
+
+            # Extract data coordinates - these are pixels in the reference image
+            x = data['domain']['x']
+            y = data['domain']['y']
+
+            if x is None or y is None:  # Out of bounds
+                self.label_mouseover.pixel = ""
+                self.label_mouseover.reset_coords_display()
+                self.label_mouseover.value = ""
+                return
+
+            maxsize = int(np.ceil(np.log10(np.max(image.shape)))) + 3
+            fmt = 'x={0:0' + str(maxsize) + '.1f} y={1:0' + str(maxsize) + '.1f}'
+            self.label_mouseover.pixel = (fmt.format(x, y))
+
+            if data_has_valid_wcs(image, ndim=2):
+                try:
+                    coo = image.coords.pixel_to_world(x, y).icrs
+                except Exception:  # WCS might not be celestial  # pragma: no cover
+                    self.label_mouseover.reset_coords_display()
+                else:
+                    self.label_mouseover.set_coords(coo)
+            else:  # pragma: no cover
+                self.label_mouseover.reset_coords_display()
+
+            # Extract data values at this position.
+            # TODO: for now we just use the first visible layer but we should think
+            # of how to display values when multiple datasets are present.
+            if (x > -0.5 and y > -0.5
+                    and x < image.shape[1] - 0.5 and y < image.shape[0] - 0.5
+                    and hasattr(active_layer, 'attribute')):
+                attribute = active_layer.attribute
+                value = image.get_data(attribute)[int(round(y)), int(round(x))]
+                unit = image.get_component(attribute).units
+                self.label_mouseover.value = f'{value:+10.5e} {unit}'
+            else:
+                self.label_mouseover.value = ''
+
+        elif data['event'] == 'mouseleave' or data['event'] == 'mouseenter':
+
+            self.label_mouseover.pixel = ""
+            self.label_mouseover.reset_coords_display()
+            self.label_mouseover.value = ""
+
     def data(self, cls=None):
         return [layer_state.layer  # .get_object(cls=cls or self.default_class)
                 for layer_state in self.state.layers
@@ -128,11 +156,9 @@ class MosvizProfile2DView(JdavizViewerMixin, BqplotImageView):
             "spectrum_2d_viewer_reference_name", "spectrum-2d-viewer"
         )
 
-        if self.jdaviz_app.config == 'specviz2d':
-            # pass
-            self.label_mouseover = None
-            self.add_event_callback(self.on_mouse_or_key_event,
-                                    events=['mousemove', 'mouseenter', 'mouseleave'])
+        self.label_mouseover = None
+        self.add_event_callback(self.on_mouse_or_key_event,
+                                events=['mousemove', 'mouseenter', 'mouseleave'])
 
     def on_mouse_or_key_event(self, data):
 
