@@ -15,9 +15,10 @@ def test_load_spectrum1d(mosviz_helper, spectrum1d):
     mosviz_helper.load_data(spectra_1d=spectrum1d, spectra_1d_label=label)
 
     assert len(mosviz_helper.app.data_collection) == 2
-    dc_0 = mosviz_helper.app.data_collection[0]
-    assert dc_0.label == label
-    assert dc_0.meta['uncertainty_type'] == 'std'
+    assert mosviz_helper.app.data_collection[0].label == "MOS Table"
+    dc_1 = mosviz_helper.app.data_collection[1]
+    assert dc_1.label == label
+    assert dc_1.meta['uncertainty_type'] == 'std'
 
     table = mosviz_helper.app.get_viewer('table-viewer')
     table.widget_table.vue_on_row_clicked(0)
@@ -36,10 +37,11 @@ def test_load_image(mosviz_helper, mos_image):
     mosviz_helper.load_images(mos_image, data_labels=label)
 
     assert len(mosviz_helper.app.data_collection) == 2
-    dc_0 = mosviz_helper.app.data_collection[0]
-    assert dc_0.label == f"{label} 0"
-    assert PRIHDR_KEY not in dc_0.meta
-    assert dc_0.meta['RADESYS'] == 'ICRS'
+    assert mosviz_helper.app.data_collection[0].label == "MOS Table"
+    dc_1 = mosviz_helper.app.data_collection[1]
+    assert dc_1.label == f"{label} 0"
+    assert PRIHDR_KEY not in dc_1.meta
+    assert dc_1.meta['RADESYS'] == 'ICRS'
 
     table = mosviz_helper.app.get_viewer('table-viewer')
     table.widget_table.vue_on_row_clicked(0)
@@ -57,9 +59,10 @@ def test_load_spectrum_collection(mosviz_helper, spectrum_collection):
 
     # +1 for the table viewer
     assert len(mosviz_helper.app.data_collection) == len(spectrum_collection) + 1
-    dc_0 = mosviz_helper.app.data_collection[0]
-    assert dc_0.label == labels[0]
-    assert dc_0.meta['uncertainty_type'] == 'std'
+    assert mosviz_helper.app.data_collection[0].label == "MOS Table"
+    dc_1 = mosviz_helper.app.data_collection[1]
+    assert dc_1.label == labels[0]
+    assert dc_1.meta['uncertainty_type'] == 'std'
 
     table = mosviz_helper.app.get_viewer('table-viewer')
     table.select_row(0)
@@ -76,9 +79,10 @@ def test_load_list_of_spectrum1d(mosviz_helper, spectrum1d):
     mosviz_helper.load_1d_spectra(spectra, data_labels=labels)
 
     assert len(mosviz_helper.app.data_collection) == 4
-    dc_0 = mosviz_helper.app.data_collection[0]
-    assert dc_0.label == labels[0]
-    assert dc_0.meta['uncertainty_type'] == 'std'
+    assert mosviz_helper.app.data_collection[0].label == "MOS Table"
+    dc_1 = mosviz_helper.app.data_collection[1]
+    assert dc_1.label == labels[0]
+    assert dc_1.meta['uncertainty_type'] == 'std'
 
     table = mosviz_helper.app.get_viewer('table-viewer')
     table.widget_table.vue_on_row_clicked(0)
@@ -95,9 +99,10 @@ def test_load_mos_spectrum2d(mosviz_helper, mos_spectrum2d):
     mosviz_helper.load_data(spectra_2d=mos_spectrum2d, spectra_2d_label=label)
 
     assert len(mosviz_helper.app.data_collection) == 2
-    dc_0 = mosviz_helper.app.data_collection[0]
-    assert dc_0.label == label
-    assert dc_0.meta['INSTRUME'] == 'nirspec'
+    assert mosviz_helper.app.data_collection[0].label == "MOS Table"
+    dc_1 = mosviz_helper.app.data_collection[1]
+    assert dc_1.label == label
+    assert dc_1.meta['INSTRUME'] == 'nirspec'
 
     table = mosviz_helper.app.get_viewer('table-viewer')
     table.widget_table.vue_on_row_clicked(0)
@@ -152,6 +157,8 @@ def test_load_single_image_multi_spec(mosviz_helper, mos_image, spectrum1d, mos_
     spectra2d = [mos_spectrum2d] * 3
 
     image_viewer = mosviz_helper.app.get_viewer('image-viewer')
+    spec1d_viewer = mosviz_helper.app.get_viewer('spectrum-viewer')
+    spec2d_viewer = mosviz_helper.app.get_viewer('spectrum-2d-viewer')
 
     # Coordinates info panel should not crash even when nothing is loaded.
     image_viewer.on_mouse_or_key_event({'event': 'mouseover'})
@@ -159,11 +166,10 @@ def test_load_single_image_multi_spec(mosviz_helper, mos_image, spectrum1d, mos_
 
     # Test that loading is still possible after previous crash:
     # https://github.com/spacetelescope/jdaviz/issues/364
-    with pytest.raises(ValueError, match="No data found with the label 'MOS Table'"):
+    with pytest.raises(ValueError, match="incompatible with the dimensions of this data:"):
         mosviz_helper.load_data(spectra1d, spectra2d, images=[])
 
-    with pytest.warns(UserWarning, match='Could not parse metadata from input images'):
-        mosviz_helper.load_data(spectra1d, spectra2d, images=mos_image, images_label=label)
+    mosviz_helper.load_data(spectra1d, spectra2d, images=mos_image, images_label=label)
 
     assert mosviz_helper.app.get_viewer("table-viewer").figure_widget.highlighted == 0
     assert len(mosviz_helper.app.data_collection) == 8
@@ -176,14 +182,15 @@ def test_load_single_image_multi_spec(mosviz_helper, mos_image, spectrum1d, mos_
     assert len(qtable) == 3
 
     # Also check coordinates info panels for Mosviz image viewer.
-    # 1D spectrum viewer panel is already tested in Specviz.
-    # 2D spectrum viewer panel is already tested in Specviz2d.
+    # 1D spectrum viewer panel is also tested in Specviz.
+    # 2D spectrum viewer panel is also tested in Specviz2d.
 
     image_viewer.on_mouse_or_key_event({'event': 'mousemove', 'domain': {'x': 0, 'y': 0}})
     assert image_viewer.label_mouseover.pixel == 'x=000.0 y=000.0'
     assert image_viewer.label_mouseover.value == '+3.74540e-01 Jy'
     assert image_viewer.label_mouseover.world_ra_deg == '5.0297844783'
     assert image_viewer.label_mouseover.world_dec_deg == '4.9918991917'
+    assert image_viewer.label_mouseover.icon == 'a'
 
     image_viewer.on_mouse_or_key_event({'event': 'mousemove', 'domain': {'x': None, 'y': 0}})
     assert image_viewer.label_mouseover.pixel == ''
@@ -203,6 +210,21 @@ def test_load_single_image_multi_spec(mosviz_helper, mos_image, spectrum1d, mos_
     assert image_viewer.label_mouseover.world_ra_deg == ''
     assert image_viewer.label_mouseover.world_dec_deg == ''
 
+    spec2d_viewer.on_mouse_or_key_event({'event': 'mousemove', 'domain': {'x': 10, 'y': 100}})
+    assert spec2d_viewer.label_mouseover.pixel == 'x=00010.0 y=00100.0'
+    assert spec2d_viewer.label_mouseover.value == '+8.12986e-01 '
+    assert spec2d_viewer.label_mouseover.world_ra_deg == ''
+    assert spec2d_viewer.label_mouseover.world_dec_deg == ''
+    assert spec2d_viewer.label_mouseover.icon == 'b'
+
+    spec1d_viewer.on_mouse_or_key_event({'event': 'mousemove', 'domain': {'x': 7000, 'y': 170}})
+    assert spec1d_viewer.label_mouseover.pixel == '7.00000e+03, 1.70000e+02'
+    assert spec1d_viewer.label_mouseover.world_label_prefix == 'Wave'
+    assert spec1d_viewer.label_mouseover.world_ra == '6.88889e+03 Angstrom (4 pix)'
+    assert spec1d_viewer.label_mouseover.world_label_prefix_2 == 'Flux'
+    assert spec1d_viewer.label_mouseover.world_ra_deg == '1.35436e+01 Jy'
+    assert spec1d_viewer.label_mouseover.icon == 'c'
+
 
 def test_zip_error(mosviz_helper, tmp_path):
     '''
