@@ -11,7 +11,7 @@ from astropy import units as u
 from astropy.io import fits
 from astropy.nddata import CCDData, StdDevUncertainty
 from astropy.wcs import WCS
-from specutils import Spectrum1D, SpectrumCollection
+from specutils import Spectrum1D, SpectrumCollection, SpectrumList
 
 from jdaviz import __version__, Cubeviz, Imviz, Mosviz, Specviz, Specviz2d
 
@@ -143,7 +143,9 @@ def spectrum1d():
             spec_axis.value/500) * u.Jy
     uncertainty = StdDevUncertainty(np.abs(np.random.randn(len(spec_axis.value))) * u.Jy)
 
-    return Spectrum1D(spectral_axis=spec_axis, flux=flux, uncertainty=uncertainty)
+    meta = dict(header=dict(FILENAME="jdaviz-test-file.fits"))
+
+    return Spectrum1D(spectral_axis=spec_axis, flux=flux, uncertainty=uncertainty, meta=meta)
 
 
 @pytest.fixture
@@ -156,9 +158,30 @@ def spectrum_collection(spectrum1d):
     return result
 
 
-def _create_spectrum1d_cube_with_fluxunit(fluxunit=u.Jy):
+@pytest.fixture
+def multi_order_spectrum_list(spectrum1d, spectral_orders=10):
+    sc = []
+    np.random.seed(42)
 
-    flux = np.arange(16).reshape((2, 2, 4)) * fluxunit
+    for i in range(spectral_orders):
+
+        spec_axis = (np.arange(SPECTRUM_SIZE) + 6000 + i * SPECTRUM_SIZE) * u.AA
+        flux = (np.random.randn(len(spec_axis.value)) +
+                10 * np.exp(-0.002 * (spec_axis.value - 6563) ** 2) +
+                spec_axis.value / 500) * u.Jy
+        uncertainty = StdDevUncertainty(np.abs(np.random.randn(len(spec_axis.value))) * u.Jy)
+        meta = dict(header=dict(FILENAME="jdaviz-test-multi-order-file.fits"))
+        spectrum1d = Spectrum1D(spectral_axis=spec_axis, flux=flux,
+                                uncertainty=uncertainty, meta=meta)
+
+        sc.append(spectrum1d)
+
+    return SpectrumList(sc)
+
+
+def _create_spectrum1d_cube_with_fluxunit(fluxunit=u.Jy, shape=(2, 2, 4)):
+
+    flux = np.arange(np.prod(shape)).reshape(shape) * fluxunit
     wcs_dict = {"CTYPE1": "RA---TAN", "CTYPE2": "DEC--TAN", "CTYPE3": "WAVE-LOG",
                 "CRVAL1": 205, "CRVAL2": 27, "CRVAL3": 4.622e-7,
                 "CDELT1": -0.0001, "CDELT2": 0.0001, "CDELT3": 8e-11,
@@ -170,7 +193,12 @@ def _create_spectrum1d_cube_with_fluxunit(fluxunit=u.Jy):
 
 @pytest.fixture
 def spectrum1d_cube():
-    return _create_spectrum1d_cube_with_fluxunit(u.Jy)
+    return _create_spectrum1d_cube_with_fluxunit(fluxunit=u.Jy)
+
+
+@pytest.fixture
+def spectrum1d_cube_larger():
+    return _create_spectrum1d_cube_with_fluxunit(fluxunit=u.Jy, shape=(SPECTRUM_SIZE, 2, 4))
 
 
 @pytest.fixture
